@@ -1,11 +1,9 @@
-# $Id: ListenAccept.pm 2347 2008-06-01 18:40:12Z rcaputo $
-
 package POE::Wheel::ListenAccept;
 
 use strict;
 
 use vars qw($VERSION);
-$VERSION = do {my($r)=(q$Revision: 2347 $=~/(\d+)/);sprintf"1.%04d",$r};
+$VERSION = '1.280'; # NOTE - Should be #.### (three decimal places)
 
 use Carp qw( croak carp );
 use Symbol qw( gensym );
@@ -13,6 +11,7 @@ use Symbol qw( gensym );
 use POSIX qw(:fcntl_h);
 use Errno qw(EWOULDBLOCK);
 use POE qw( Wheel );
+use base qw(POE::Wheel);
 
 sub SELF_HANDLE       () { 0 }
 sub SELF_EVENT_ACCEPT () { 1 }
@@ -354,3 +353,4 @@ Please see L<POE> for more information about authors and contributors.
 =cut
 
 # rocco // vim: ts=2 sw=2 expandtab
+# TODO - Edit.
